@@ -40,12 +40,15 @@ from gym.utils import seeding
 
 from . import veinsgym_pb2
 
+SENTINEL_EMPTY_SPACE = gym.spaces.Space()
+SENTINEL_NO_SEED_GIVEN = "NO SEED GIVEN"  # replace with pep 661 once ready
+
 
 class StepResult(NamedTuple):
     """Result record from one step in the invironment."""
 
     observation: Any
-    reward: float
+    reward: np.float32
     done: bool
     info: Dict
 
@@ -66,7 +69,13 @@ def ensure_valid_scenario_dir(scenario_dir):
 
 
 def launch_veins(
-    scenario_dir, seed, port, print_stdout=False, extra_args=None
+    scenario_dir,
+    seed,
+    port,
+    print_stdout=False,
+    extra_args=None,
+    user_interface="Cmdenv",
+    config="General",
 ):
     """
     Launch a veins experiment and return the process instance.
@@ -76,7 +85,8 @@ def launch_veins(
     """
     command = [
         "./run",
-        "-uCmdenv",
+        f"-u{user_interface}",
+        f"-c{config}",
         f"--seed-set={seed}",
         f"--*.manager.seed={seed}",
         f"--*.gym_connection.port={port}",
@@ -112,6 +122,12 @@ def shutdown_veins(process, gracetime_s=1.0):
             process.pid,
         )
         process.kill()
+        try:
+            process.wait(gracetime_s)
+        except subprocess.TimeoutExpired as _exc2:
+            logging.error(
+                "Veins process %d could not even be killed!", process.pid
+            )
     assert (
         process.poll() and process.returncode is not None
     ), "Veins could not be killed."
@@ -129,7 +145,7 @@ def parse_space(space):
     if space.HasField("discrete"):
         return space.discrete.value
     if space.HasField("box"):
-        return np.array(space.box.values, dtype=float)
+        return np.array(space.box.values, dtype=np.float32)
     if space.HasField("multi_discrete"):
         return np.array(space.multi_discrete.values, dtype=int)
     if space.HasField("multi_binary"):
@@ -160,6 +176,8 @@ class VeinsEnv(gym.Env):
         print_veins_stdout=False,
         action_serializer=serialize_action_discete,
         veins_kwargs=None,
+        user_interface="Cmdenv",
+        config="General",
     ):
         if scenario_dir is None:
             scenario_dir = self.default_scenario_dir
@@ -167,8 +185,8 @@ class VeinsEnv(gym.Env):
         self.scenario_dir = scenario_dir
         self._action_serializer = action_serializer
 
-        self.action_space = None
-        self.observation_space = None
+        self.action_space = SENTINEL_EMPTY_SPACE
+        self.observation_space = SENTINEL_EMPTY_SPACE
 
         self.context = zmq.Context()
         self.socket = self.context.socket(zmq.REP)
@@ -181,6 +199,8 @@ class VeinsEnv(gym.Env):
         self._passed_args = (
             veins_kwargs if veins_kwargs is not None else dict()
         )
+        self._user_interface = user_interface
+        self._config = config
         self._seed = 0
         self.veins = None
         self._veins_shutdown_handler = None
@@ -203,18 +223,22 @@ class VeinsEnv(gym.Env):
             self.socket.send(
                 self._action_serializer(self.action_space.sample())
             )
-            logging.debug("Episode ended, waiting for veins to finish")
-            self.veins.wait()
+            logging.debug("Episode ended")
+            if self.veins:
+                logging.debug("Waiting for veins to finish")
+                self.veins.wait()
         assert self.observation_space.contains(step_result.observation)
         return step_result
 
-    def reset(self):
+    def reset(self, seed=SENTINEL_NO_SEED_GIVEN, return_info=False, options=None):
         """
         Start and connect to a new veins experiment, return first observation.
 
         Shut down exisiting veins experiment processes and connections.
         Waits until first request from veins experiment has been received.
         """
+        del options  # currently not used/implemented
+
         self.close()
         self.socket = self.context.socket(zmq.REP)
         if self.port is None:
@@ -227,6 +251,9 @@ class VeinsEnv(gym.Env):
             self.bound_port = self.port
             logging.debug("Listening on configured port %d", self.bound_port)
 
+        if seed is not SENTINEL_NO_SEED_GIVEN:
+            self.seed(seed)
+
         if self.run_veins:
             self.veins = launch_veins(
                 self.scenario_dir,
@@ -234,6 +261,8 @@ class VeinsEnv(gym.Env):
                 self.bound_port,
                 self.print_veins_stdout,
                 self._passed_args,
+                self._user_interface,
+                self._config,
             )
             logging.info("Launched veins experiment, waiting for request.")
 
@@ -255,6 +284,11 @@ class VeinsEnv(gym.Env):
 
         initial_request = self._parse_request(self._recv_request())[0]
         logging.info("Received first request from Veins, ready to run.")
+        if return_info:
+            logging.warning("return info not yet implemented for reset()")
+            initial_info = None  # not implemented
+            return initial_request, initial_info
+
         return initial_request
 
     def render(self, mode="human"):
@@ -336,6 +370,9 @@ class VeinsEnv(gym.Env):
             real_request.ParseFromString(real_data)
             # continue processing the real request
             request = real_request
+        # the gym needs to be initialized at this point!
+        assert self.action_space is not SENTINEL_EMPTY_SPACE
+        assert self.observation_space is not SENTINEL_EMPTY_SPACE
         observation = parse_space(request.step.observation)
         reward = parse_space(request.step.reward)
         assert len(reward) == 1
